(* Location: gestion des positions et messages d'erreurs *)

module Location =
  struct
    type t = int * int
    let pos() = (Parsing.symbol_start(),Parsing.symbol_end())
    let npos n = (Parsing.rhs_start(n),Parsing.rhs_end(n))
    let dummy = (-1,-1) (* pour les expressions qui ne viennent pas de la source, comme 0-x pour -x *)
    let stringpos (x,y) = "position: "^(string_of_int x)^" - "^(string_of_int y)
  end

(* Symbol: tables des symboles : interface et implementation *)

module type SYMBOL =
sig
   type symbol
   type 'a table
   val symbol: string -> symbol
   val name: symbol -> string
   val mkempty : unit -> 'a table
   val add : symbol -> 'a -> 'a table -> 'a table
   val find : symbol -> 'a table -> 'a
   val look : symbol -> 'a table -> 'a option
   val addlist : (symbol * 'a) list -> 'a table -> 'a table
   val maketable : (symbol * 'a) list -> 'a table
end

module Symbol : SYMBOL =
  struct
    type symbol = string
    let symbol n = n
    let name s = s
    module Assl = Map.Make(struct type t = symbol let compare = compare end)
    type 'a table = 'a Assl.t
    let mkempty () = Assl.empty
    let add k v table = Assl.add k v table
    let find n table = Assl.find n table
    let look n table = try Some(Assl.find n table) with Not_found -> None
    let addlist l t = List.fold_left (fun t -> fun (k,v) -> add k v t) t l
    let maketable l = addlist l (mkempty ())
  end

(* Absyn: la syntaxe abstraite de CTigre *)
module Absyn =
  struct
    type symbol = Symbol.symbol
    type core_type = 
        { typ_desc: core_type_desc;
          typ_loc: Location.t}
    and  core_type_desc =
         Typ_name of symbol
      |  Typ_arrow of core_type * core_type
      |  Typ_array of symbol
      |  Typ_record of field list
    and  type_dec = (symbol * core_type)

    and exp = 
        { exp_desc: exp_desc;
          exp_loc: Location.t}
    and forexp = {var: symbol; lo: exp; hi: exp; dir: direction_flag; for_body: exp}
    and direction_flag = Up | Down
    and arrayexp = {a_typ: symbol; size: exp; a_init: exp}
    and attribvar = {vid: symbol}
    and var = SimpleVar of attribvar
            | FieldVar of var * symbol
            | SubscriptVar of var * exp
    and  exp_desc =
        VarExp of var
      | NilExp
      | IntExp of int
      | StringExp of string
      | CharExp of char
      | Apply of symbol * exp list 
      | RecordExp of ((symbol * exp) list) * symbol
      | SeqExp of exp * exp
      | IfExp of exp * exp * exp option
      | WhileExp of exp * exp
      | ForExp of forexp
      | LetExp of dec list * exp
      | TypeExp of type_dec list * exp
      | ArrayExp of arrayexp
      | Opexp of oper * exp * exp
      | AssignExp of var * exp

    and oper = PlusOp | MinusOp | TimesOp | DivideOp
         | AndOp | OrOp 
         | EqOp | NeqOp | LtOp | LeOp | GtOp | GeOp

    and dec = 
        FunDec of fundec list
      | VarDec of vardec list

    and field = {f_name: symbol; 
             typ: symbol;
             f_loc: Location.t}
    and   fundec = {fun_name: symbol;
         	params: field list;
		result: symbol option;
		body: exp;
		fun_loc: Location.t}
    and vardec = {v_name: symbol; 
                  var_typ: symbol option;
		  init: exp; var_pos: Location.t}
   (* et une petite fonction pour afficher la position *)
   let exppos = function {exp_loc=p} -> Location.stringpos p
end



