open Absyntax
open Absyn

module PrintAbsyn : 
     sig val print : out_channel * Absyn.exp -> unit end =
struct

  module A = Absyn (* pour ne pas taper Absyn toutes les fois *)

let print (outstream, e0) =
 let say s =  output_string outstream s
 in let sayln s= (say s; say "\n") 
 in let rec indent = function 0 -> ()
                     |        i -> (say " "; indent(i-1))
 in let opname = function
      A.PlusOp -> "PlusOp"
    | A.MinusOp -> "MinusOp"
    | A.TimesOp -> "TimesOp"
    | A.DivideOp -> "DivideOp"
    | A.EqOp -> "EqOp"
    | A.NeqOp -> "NeqOp"
    | A.LtOp -> "LtOp"
    | A.LeOp -> "LeOp"
    | A.GtOp -> "GtOp"
    | A.GeOp -> "GeOp"
    | A.AndOp -> "AndOp"
    | A.OrOp -> "OrOp"

  in let rec dolist d f = function
      [a] -> (sayln ""; f(a,d+1))
    | (a::r) -> (sayln ""; f(a,d+1); say ","; dolist d f r)
    | [] -> ()


  in let rec  var = function
      (A.SimpleVar({vid=s}),d) -> (indent d; say "SimpleVar("; 
			         say(Symbol.name s); say ")")
    | (A.FieldVar(v,s),d) -> (indent d; sayln "FieldVar(";
				  var(v,d+1); sayln ",";
				  indent(d+1); say(Symbol.name s); say ")")
    | (A.SubscriptVar(v,e),d) -> (indent d; sayln "SubscriptVar(";
				      var(v,d+1); sayln ",";
				      exp(e,d+1); say ")")
  and exp = function
      ({exp_desc=A.VarExp v}, d) -> (indent d; sayln "VarExp("; var(v,d+1); say ")")
    | ({exp_desc=A.NilExp}, d) -> (indent d; say "NilExp")
    | ({exp_desc=A.IntExp i}, d) -> (indent d; say "IntExp("; say(string_of_int i);
			    say ")")
    | ({exp_desc=A.StringExp(s)},d) -> (indent d; say "StringExp(\"";
             		 say s; say "\")")
    | ({exp_desc=A.Apply(s,args)},d) ->
                     (indent d; say "Call("; say(Symbol.name s);
             	 say ",["; dolist d exp args; say "])")
    | ({exp_desc=A.Opexp(oper,left,right)},d) ->
             (indent d; say "OpExp("; say(opname oper); sayln ",";
              exp(left,d+1); sayln ","; exp(right,d+1); say ")")
    | ({exp_desc=A.RecordExp(fields,typ)},d) ->
         let f((name,e),d) =
             	(indent d; say "("; say(Symbol.name name);
             	 sayln ","; exp(e,d+1);
             	 say ")")
          in indent d; say "RecordExp("; say(Symbol.name typ); 
             sayln ",["; dolist d f fields; say "])" 
    | ({exp_desc=A.SeqExp(e1,e2)}, d) -> (indent d; say "SeqExp["; dolist d exp [e1;e2]; 
             	    say "]")
    | ({exp_desc=A.AssignExp(v,e)},d) -> 
             (indent d; sayln "AssignExp("; var(v,d+1); sayln ",";
              exp(e,d+1); say ")")
    | ({exp_desc=A.IfExp(test,then',else')},d) ->
             (indent d; sayln "IfExp("; exp(test,d+1); sayln ",";
              exp(then',d+1);
              match else' with None -> ()
             	| Some e -> (sayln ","; exp(e,d+1));
              say ")")
    | ({exp_desc=A.WhileExp(test,body)},d) ->
             (indent d; sayln "WhileExp("; exp(test,d+1); sayln ",";
              exp(body,d+1); say ")")
    | ({exp_desc=A.ForExp{var=v;lo=lo;hi=hi;dir=dir;for_body=body}},d) ->
             (indent d; sayln "ForExp(";
              say(Symbol.name v); sayln ",";
              exp(lo,d+1); sayln ","; exp(hi,d+1); sayln ",";
              exp(body,d+1); say ")")
    | ({exp_desc=A.ArrayExp{a_typ=typ;size=size;a_init=init}},d) ->
	        (indent d; say "ArrayExp("; say(Symbol.name typ); sayln ",";
		 exp(size,d+1); sayln ","; exp(init,d+1); say ")")
    | ({exp_desc=A.LetExp(decs,body)},d) ->
             (indent d; say "LetExp([";
              dolist d dec decs; sayln "],"; exp(body,d+1); say")")
    | ({exp_desc=A.TypeExp(tdecs,body)},d) ->
	    let tdec((name,t),d) = (indent d; say"("; 
		                               say(Symbol.name name); sayln ",";
		                               ty(t,d+1); say ")")
	    in  (indent d; say "TypeExp([";
                 dolist d tdec tdecs; sayln "],"; exp(body,d+1); say")")

  and dec = function
      (A.FunDec l, d) -> 
	    let field ({f_name=name;typ=typ},d) =
			(indent d; say "("; say(Symbol.name name);
			 say ","; say(Symbol.name typ); say ")")
	    in let f({fun_name=name;params=params;result=result;body=body},d) =
		   (indent d; say "("; say (Symbol.name name); say ",[";
		    dolist d field params; sayln "],";
		    match result with None -> say "None"
			 | Some(s) -> (say "Some("; say(Symbol.name s); say ")");
		    sayln ","; exp(body,d+1); say ")")
            in indent d; say "FunctionDec["; dolist d f l; say "]"
    | (A.VarDec vl,d) ->
           let pvar ({v_name=name;var_typ=typ;init=init},d) =
	   (indent d; say "VarDec("; say(Symbol.name name); say ",";
	    match typ with None -> say "None" 
		      | Some(s)-> (say "Some("; say(Symbol.name s); say ")");
            sayln ","; exp(init,d+1); say ")")
           in dolist d pvar vl
   
  and ty = function
      ({typ_desc=A.Typ_name(s)}, d) -> (indent d; say "Typ_name("; say(Symbol.name s);
			      say ")")
    | ({typ_desc=A.Typ_record l}, d) ->  
		let f({f_name=name;typ=typ},d) =
			(indent d; say "("; say (Symbol.name name);
			 say (Symbol.name typ); say ")")
	         in indent d; say "Typ_record["; dolist d f l; say "]"
    | ({typ_desc=A.Typ_array(s)},d) -> (indent d; say "Typ_array("; say(Symbol.name s);
			      say ")")

 in  exp(e0,0); sayln ""; flush outstream
end

